/*
 * Copyright (c) 2009 by Internet Systems Consortium, Inc. ("ISC")
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
 * OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/* Import. */

#include "nmsg.h"
#include "private.h"

/* Globals. */

bool			_nmsg_global_autoclose = true;
int			_nmsg_global_debug;
struct nmsg_msgmodset *	_nmsg_global_msgmodset;

/* Statics. */

static int		_nmsg_initialized = 0;

/* Forward. */

static void _nmsg_fini(void);

/* Export. */

nmsg_res
nmsg_init(void) {
	assert(_nmsg_initialized == 0);

	_nmsg_global_msgmodset = _nmsg_msgmodset_init(NULL);
	if (_nmsg_global_msgmodset == NULL)
		return (nmsg_res_failure);
	atexit(_nmsg_fini);

	_nmsg_alias_init();

	_nmsg_initialized = 1;
	return (nmsg_res_success);
}

void
nmsg_set_autoclose(bool autoclose) {
	_nmsg_global_autoclose = autoclose;
}

void
nmsg_set_debug(int debug) {
	_nmsg_global_debug = debug;
}

/* Private. */

void
_nmsg_fini(void) {
	_nmsg_msgmodset_destroy(&_nmsg_global_msgmodset);
	_nmsg_alias_fini();
}
